<?php
	define('CACHE_PATH',   '.cache');
	define('CACHE_CHMOD',  0775);
	define('IMAGE_PATH',   'pngs/');
	define('ERR_ID_NOT_FOUND', 'E1');

	$sheet_id=strtolower($_GET['sheetid']);
	$mask_color = -1;
	$quality=-1;
	$stack=-1;
	$sap = -1;
	$destroyed=-1;

	if(isset($_GET['q'])){
		$quality =(int) $_GET['q'];
	}
	if(isset($_GET['stack'])){
		$stack   =(int) $_GET['stack'];
	}
	if(isset($_GET['sap'])){
		$sap = ((int) $_GET['sap']) -1; // sap=0 - no sap icon, sap=1 - sap 0 is displayed on image
	}
	if(isset($_GET['destroyed']) && $_GET['destroyed'] == 1){
		$destroyed = 1;
	}

	if(isset($_GET['color'])){
		$idx=(int)$_GET['color'];
		$color_array=array(
			0 => array(233,22,0,255),
			1 => array(220,140,50,255),
			2 => array(170,250,0,255),
			3 => array(0,215,120,255),
			4 => array(50,100,255,255),
			5 => array(170,55,110,255),
			6 => array(250,250,250,255),
			7 => array(80,80,120,255),
		);
		if(isset($color_array[$idx])){
			$mask_color = $color_array[$idx][0] + ($color_array[$idx][1] << 8) + ($color_array[$idx][2] << 16); 
		}
	}

	//
	$cache_file = md5($sheet_id.$mask_color.$quality.$stack.$sap.$destroyed).'.png';
	// if cached file is found, this function will redirect to cached file and terminates
	if(($cache=cache_check($cache_file))!==false){
		// now check if item.php, items database or image folder modified time has changed
		$valid=true;
		$cache_mtime=filemtime($cache);
		$checkFiles = array('item.php', 'items_db.inc.php', IMAGE_PATH.'/.');
		foreach($checkFiles as $file){
			if(filemtime($file) > $cache_mtime){
				$valid = false;
			}
		}
		// display cached file if all checks out, if not, regenerate the file
		if($valid) cache_display($cache);
	}

	require_once('items_db.inc.php');
	if(!isset($itemsList[$sheet_id])){
		die(ERR_ID_NOT_FOUND);
	}
	$item=$itemsList[$sheet_id];

	$im=create_image($item, $mask_color, $quality, $stack, $sap, $destroyed);
	if(($cache=cache_save($cache_file, $im))!==false){
		cache_display($cache);
	}

	// caching not possible, show the image
	header('Content-Type: image/png');
	imagepng($im);
	imagedestroy($im);
	die();	

function create_image($item, $mask_color, $quality, $stack, $sap, $destoryed){
	list($bgFile, $bgColor)=split_fname($item['bg']);
	list($fgFile, $fgColor)=split_fname($item['fg']);
	list($i1File, $i1Color)=split_fname($item['i1']);
	list($i2File, $i2Color)=split_fname($item['i2']);

	// 'bk_generic.png' as common background to make semi transparent backgrounds little 'darker'
	$bg = load_image(IMAGE_PATH.'bg_empty.png', -1, $mask_color);
	if($bg===false){
		$bg=imagecreatetruecolor(40, 40);
		//$trans=imagecolorallocatealpha($bg, 0, 0, 0, 127);
		//imagefill($bg, 0, 0, $trans);
		//imagesavealpha($bg, true);
		//imagecolortransparent($bg, $trans);
		$gray = imagecolorallocate($bg, 0xdd, 0xdd, 0xdd);
		imagefill($bg, 0, 0, $gray);
	}

	// get background image
	if($bgFile!='') {
		$bg_real = load_image(IMAGE_PATH.$bgFile, $bgColor, $mask_color);
		if($bg_real!==false){
			imagecopy($bg, $bg_real, 0,0,0,0,40,40);
			imagedestroy($bg_real);
		}
	}
	// foreground image
	if($fgFile!='') {
		$fg = load_image(IMAGE_PATH.$fgFile, $fgColor, $mask_color);
		if($fg!==false){
			imagecopy($bg, $fg, 0, 0, 0, 0, 40, 40);
			imagedestroy($fg);
		}
	}
	//
	if($i1File!=''){
		$im=load_image(IMAGE_PATH.$i1File, $i1Color, $mask_color);
		if($im!==false){
			imagecopy($bg, $im, 0, 0, 0, 0, 40, 40);
			imagedestroy($im);
		}
	}
	//
	if($i2File!=''){
		$im=load_image(IMAGE_PATH.$i2File, $i2Color);
		if($im!==false){
			imagecopy($bg, $im, 0, 0, 0, 0, 40, 40);
			imagedestroy($im);
		}
	}

	// put item name, quality, stack size and sap loar to final image
	if($item['txt']!=''){ // top right
		image_typo($bg, $item['txt'], 1, 0);
	}
	if($quality>0){ // to bottom-right corner
		image_typo($bg, $quality, 40-strlen($quality)*5, 32);// 40-8
	}
	if($stack>0){ // bottom-left corner
		image_typo($bg, 'x'.$stack, 1, 32);// 40-8
	}
	if($sap>0){ // top-right corenr
		$im=load_image(IMAGE_PATH.'sapload.png');
		if($im!==false){
			imagecopy($bg, $im, 0, 0, 0, 0, 40, 40);
			imagedestroy($im);
			image_typo($bg, $sap, 1,2);
		}
	}
	if($destroyed==1){
		$im=load_image(IMAGE_PATH.'ico_task_failed.png');
		if($im!==false){
			imagecopy($bg, $im, 0, 0, 0, 0, 40, 40);
			imagedestroy($im);
		}
	}
	//imagesavealpha($bg, false);
	return $bg;
} // create_image
//-EOF

// return image handler, colorize if needed, apply mask if needed and mask color is set
function load_image($fileName, $color=-1, $mColor=-1){
	$im = @imagecreatefrompng($fileName);
	if($im===false){
		return false;
	}
	imagesavealpha($im, true);

	if($color != -1) {
		$im=colorize($im, ($color >> 24) & 255, $color&255, ($color>>8)&255, ($color >> 16)&255);
	}elseif($rzColor != -1){
		$mask_file=str_replace('.png', '_mask.png', $fileName);
		if(file_exists($mask_file)){
			$mask = imagecreatefrompng($mask_file);
			if($mask!==false){
				imagesavealpha($mask, true);
				$mask=colorize($mask, ($mColor >> 24) & 255, $mColor&255, ($mColor>>8)&255, ($mColor >> 16)&255);
				imagecopy($im, $mask, 0, 0, 0, 0, 40, 40);
			}
		}
	}
	return $im;
}

function colorize($im, $ca, $cr, $cg, $cb){
	$w=imagesx($im);
	$h=imagesx($im);

	$out = imagecreatetruecolor($w, $h);
	$trans=imagecolorallocatealpha($out, 0, 0, 0, 127);
	imagefill($out, 0, 0, $trans);
	imagesavealpha($out, true);

	for($x=0;$x<$w;$x++){
		for($y=0;$y<$h;$y++){
			$rgba=imagecolorsforindex($im, imagecolorat($im, $x, $y));
			$r=$cr*$rgba['red']/255;
			$g=$cg*$rgba['green']/255;
			$b=$cb*$rgba['blue']/255;
			$c=imagecolorallocatealpha($out, $r, $g, $b, $rgba['alpha']);
			imagesetpixel($out, $x, $y, $c);
		}
	}

	return $out;
}

function image_typo($bg, $txt, $x, $y){
	for($i=0;$i<strlen($txt);$i++){
		if($txt{$i}=='?') $t='question'; else $t=strtolower(substr($txt, $i, 1));
		if(is_numeric($t)){
			$typo_file=IMAGE_PATH.'typo/numbers_'.$t.'.png';
		}else{
			$typo_file=IMAGE_PATH.'typo/typo_'.$t.'.png';
		}
		if(file_exists($typo_file)){
			$im=imagecreatefrompng($typo_file);
			$w=imagesx($im);
			$h=imagesy($im);
			imagecopy($bg, $im, $x+$i*5, $y+(8-$h)/2, 0, 0, $w, $h); // center 'char' to 5x8
		}
	}
}

function split_fname($fname){
	// get image names and possible colors
	if(strpos($fname, '|')===false){
		return array($fname, -1);
	}else{
		return explode('|', $fname);
	}
}

// return cache path
function cache_path($cache_file){
	$_l1=substr($cache_file, 0, 1);
	$_l2=substr($cache_file, 1, 1);
	return CACHE_PATH.'/'.$_l1.'/'.$_l2;
}
// 
function cache_save($cache_file, $im){
	$path=cache_path($cache_file);

	// chech the directory
	if(!file_exists($path)){
		// recursively make directories - requires php 5
		if(!@mkdir($path, CACHE_CHMOD, true)){
			return false;
		}
	}else if(!is_dir($path)){
		return false;
	}

	$fullPath=$path.'/'.$cache_file;
	imagepng($im, $fullPath);
	imagedestroy($im);
	return $fullPath;
}
//
function cache_check($cache_file){
	$fullPath = cache_path($cache_file).'/'.$cache_file;
	if($_SERVER['Pragma'] == 'no-cache' || $_SERVER['Cache-Control'] == 'no-cache' || !file_exists($fullPath)){
		return false;
	}
	return $fullPath;
}
function cache_display($file){
    $if_none_match = $_SERVER['HTTP_IF_NONE_MATCH'];// => "92744f5cc45cd2ef39816d58c5999d3c"
    $if_modified_since= $_SERVER['HTTP_IF_MODIFIED_SINCE'];// => Sat, 11 Apr 2009 14:36:36 GMT

	$size  = filesize($file);
	$mtime = filemtime($file);

	$last_modified=gmstrftime("%a, %d %b %Y %T %Z", $mtime);
	$etag = '"'.md5($file).'"';
	
	// if this is enabled, helf the time connection dies out with HTTP/1.1 200 (and no image)
	/*if( $if_none_match == $etag && $if_modified_since == $last_modified){
		header('HTTP/1.1 304 Not Modified');
		header('Cache-Control: private');
		header('Pragma: ');
		header('Expires: ');
		header('Content-Type: ');
		header('Content-Length: 0');
		header('ETag: '.$etag);
		ob_clean();
		flush();
		exit;
	}else*/{
		header('Content-Type: image/png');
		header('Content-Length: '.$size);
		header('Cache-Control: private');
		header('Pragma: ');
		header('Expires: ');
		header('Last-Modified: '.$last_modified);
		header('ETag: '.$etag);
		ob_clean();
		flush();
		readfile($file);
		exit;
	}
}
